package caching.logging;

import static org.junit.Assert.assertEquals;

import java.util.Iterator;
import java.util.Set;

import org.junit.Test;

import caching.dao.CustomerDao;
import caching.dao.DaoFactory;

public class TestLogAnalyser {
	@Test
	public void canReportNumberOfEntries() {
		InvocationLogger logger = createInvocationLoggerWithTenTestEntries();
		LogAnalyzer analyzer = new LogAnalyzer(logger);
		assertEquals("Expected 10 entries",10, analyzer.reportNumberOfEntries());
	}
	private InvocationLogger createInvocationLoggerWithTenTestEntries() {
		InvocationLogger logger = InvocationLogger.getInstance();
		for (int i = 1; i <= 10; i++) {
			logger.log(InvocationLogEntry.createEntry("entry"+i, 0));
		}
		return logger;
	}
	@Test
	public void canReportNumberOfUniqueEntries() {
		LogAnalyzer analyzer = createLogAnalyzerWithTwoUniqueEntries();
		assertEquals("Expected to find 2 unique entries", 2, analyzer.reportNumberOfUniqueEntries());
	}

	@Test
	public void canReportNumberOfOccurencesForAnEntry() {
		LogAnalyzer analyzer = createLogAnalyzerWithTwoUniqueEntries();
		assertEquals("Expected to get a count of 2 for entry1",2, analyzer.reportNumberOfOccurencesForEntry(InvocationLogEntry.createEntry("entry1", 0)));
		assertEquals("Expected to get a count of 1 for entry2",1, analyzer.reportNumberOfOccurencesForEntry(InvocationLogEntry.createEntry("entry2", 0)));
	}
	private LogAnalyzer createLogAnalyzerWithTwoUniqueEntries() {
		InvocationLogger logger = InvocationLogger.getInstance();
		logger.log(InvocationLogEntry.createEntry("entry1", 0));
		logger.log(InvocationLogEntry.createEntry("entry2", 0));
		logger.log(InvocationLogEntry.createEntry("entry1", 0));
		LogAnalyzer analyzer = new LogAnalyzer(logger);
		return analyzer;
	}
	@Test
	public void canReportNumberOfOccurencesIfNoEntryWasFound() {
		InvocationLogger logger = InvocationLogger.getInstance();
		LogAnalyzer analyzer = new LogAnalyzer(logger);
		assertEquals("Expected to get a count of 0 for entry1",0, analyzer.reportNumberOfOccurencesForEntry(InvocationLogEntry.createEntry("entry1", 0)));
	}
	@Test
	public void canReportAverageResponseTimeForMultipleInvocationsOfOneMethod() {
		InvocationLogger logger = InvocationLogger.getInstance();
		InvocationLogEntry entry1 = InvocationLogEntry.createEntry("methodname", new Object[]{"arg1"} , "retrunval1", 1);
		InvocationLogEntry entry2 = InvocationLogEntry.createEntry("methodname", new Object[]{"arg2"} , "retrunval2", 2);
		InvocationLogEntry entry3 = InvocationLogEntry.createEntry("methodname", new Object[]{"arg3"} , "retrunval3", 5);
		InvocationLogEntry entry4 = InvocationLogEntry.createEntry("methodname2", new Object[]{"arg4"} , "retrunval5", 5);
		logger.log(entry1);
		logger.log(entry2);
		logger.log(entry3);
		logger.log(entry4);
		LogAnalyzer analyzer = new LogAnalyzer(logger);
		assertEquals("Average for methodname should be 2", 2.66D, analyzer.reportAverageExceutionTimeByMethodName("methodname"), 0.01D);
	}
	@Test
	public void canReportUniqueEntriesByOccurence() {
		InvocationLogger logger = InvocationLogger.getInstance();
		CustomerDao dao = DaoFactory.getInstance(logger).getCustomerDao();
		dao.findCustomerById("customerid1");
		dao.findCustomerById("customerid2");
		dao.findCustomerById("customerid3");
		dao.findCustomerById("customerid3");
		dao.findCustomerById("customerid4");
		dao.findCustomerById("customerid2");
		dao.findCustomerById("customerid5");
		dao.findCustomerById("customerid3");
		
		LogAnalyzer analyzer = new LogAnalyzer(logger);
		assertEquals(5,analyzer.reportNumberOfEntries());
		assertEquals(5,analyzer.reportNumberOfUniqueEntries());
		Set<InvocationLogEntry> uniqueEntries = analyzer.reportUniqueInvocationEntries();
		assertEquals(5,uniqueEntries.size());
		Iterator<InvocationLogEntry> iterator = uniqueEntries.iterator();
		while (iterator.hasNext()) {
			InvocationLogEntry invocationLogEntry = (InvocationLogEntry) iterator.next();
			int numberOfOccurences = analyzer.reportNumberOfOccurencesForEntry(invocationLogEntry);
			System.out.println(invocationLogEntry +" occured "+numberOfOccurences);
		}
	}
}
